#!/bin/bash

PARTBOOT=1
PARTSWAP=2
PARTROOT=3
getuuid()
{
    local dev=$1
    local rv=`blkid  $dev -o value|grep '^........-....-....-....-............$'`
    echo "$rv"
}
clonaruuid()
{
    local orig=$1
    local dest=$2
    local part=$3
    uuid=$( getuuid /dev/${orig}${part} )
    if [ -n "$uuid" ];then
        fsck -fyC0 /dev/${dest}${part}
        flush
        echo $TUNEFS -U "$uuid" /dev/${dest}${part}
        $TUNEFS -U "$uuid" /dev/${dest}${part}
    fi
}
checkerr()
{
    local code=$1
    if [ $code -ne 0 ];then
        echo "error $code"
        exit $code
    fi
}
flush()
{
    sleep 1
    sync
    sleep 1
}
clonarefi()
{
    echo "Clono ${discodest}${PARTEFI}"
    dd if=/dev/${discoorig}${PARTEFI} of=/dev/${discodest}${PARTEFI} bs=1M
    fsck -fyC0 /dev/${discodest}${PARTEFI}
}
sincronizarpart()
{
    part=/dev/$1
    orig=$2
    dest=/mnt/000temp$1$RANDOM/
    echo "Rsync $orig"
    if [ ! -e $part ];then
        echo "$part no existe"
        exit -2
    fi
    umount $part 2>/dev/null
    flush
    mkdir $dest
    mount $part $dest
    checkerr $?
    mount 2>&1 |grep "^${part} .*type.ext4" -q && EXT4=1 || EXT4=0
    if  [ $EXT4 -eq 1 ]; then
        fstrim -v $dest
        flush
    fi
    mount 2>&1 |grep "^${part} .*type.vfat" -q && VFAT=1 || VFAT=0
    EXTENDEDATTRS=X
    if  [ $VFAT -eq 1 ]; then
        EXTENDEDATTRS=
    fi
    local PROGRESS_OPTION=
    local VERBATTRS=q
    if  [ $RSYNC_PROGRESS -eq 1 ]; then
        PROGRESS_OPTION=" --info=progress2"
        VERBATTRS=
    fi
    rsync -aH${VERBATTRS}Ax${EXTENDEDATTRS} --delete --exclude '000temp*' --exclude 'adminbcm_fechaclon' ${PROGRESS_OPTION} $orig $dest
    syncerr=$?
    flush
    umount $part
    rmdir $dest
    checkerr $?
}
sincronizar()
{
    $TUNEFS -L/bootnew$RANDOM /dev/${discodest}${PARTBOOT}
    $TUNEFS -L/new$RANDOM /dev/${discodest}${PARTROOT}
    syncerr=0
    sincronizarpart ${discodest}${PARTBOOT} /boot/
    if [ $syncerr -eq 0 ];then
        sincronizarpart ${discodest}${PARTROOT} /
    fi
    if [ $syncerr -eq 0 ] && [ -n "$PARTEFI" ];then
        clonarefi
    fi
    $TUNEFS -L/boot /dev/${discodest}${PARTBOOT}
    $TUNEFS -L/ /dev/${discodest}${PARTROOT}
    checkerr $syncerr
}
particionar()
{
if [ $PARTROOT -eq 4 ];then
local disksize=$(cat /sys/block/${discodest}/size)
local growpart=0
if [ "0$disksize" -gt 10000000 ];then
    growpart=1
    disksize=+2G
else
    disksize=
fi
cat <<EOF | fdisk /dev/${discodest}
p
d
${PARTROOT}
n
p

${disksize}
w
EOF
else
cat <<EOF | fdisk /dev/${discodest}
p
d
${PARTROOT}
n
p
${PARTROOT}


w
EOF
fi
if [ "0${growpart}" -eq 1 ] && [ -x /usr/bin/growpart ];then
    growpart /dev/${discodest} ${PARTROOT}
fi
}
FORCEMKFS=
FORCETUNEFS=
RSYNC_PROGRESS=0
versioncentos=`/bin/cat /etc/redhat-release 2>/dev/null |/bin/sed 's/.*release //g;s/\..*//g;s/[^0-9]//g'`
if [ 0$versioncentos -eq 5 ];then
    MKFS=mke4fs
    TUNEFS=tune4fs
    HASH_ALG=tea
    CLONAR_UUID=0
elif [ 0$versioncentos -ge 6 ];then
    MKFS=mke2fs
    TUNEFS=tune2fs
    HASH_ALG=half_md4
    CLONAR_UUID=1
    if [ 0$versioncentos -ge 8 ];then
        FORCEMKFS="-F"
        FORCETUNEFS="-f"
        RSYNC_PROGRESS=1
    fi
else
    echo "Imposible determinar la version de centos"
    exit -2
fi
clear
cat /etc/fstab
fdisk -l
df -h
solosync=0
discoorig=sda
discodest=sdb
if [ -n "$DISCODEST" ];then
    discodest=$DISCODEST
elif [ -n "$1" ] ;then
    discodest=$1
fi
echo "DISCODEST=$discodest"
if [ ! -e /dev/${discodest} ];then
    echo "El disco de destino ${discodest} no existe"
    exit -1
fi
if fdisk -l /dev/${discoorig}|grep -Eq "${discoorig}1.*(FAT16|EFI)";then
    PARTEFI=1
    PARTBOOT=$((PARTBOOT+1))
    PARTSWAP=$((PARTSWAP+1))
    PARTROOT=$((PARTROOT+1))
fi

echo "1) Clonar particiones y sincronizar ${discodest}"
if [ -e /dev/${discodest}${PARTBOOT} ] && [ -e /dev/${discodest}${PARTROOT} ] && \
   [ ! -e /dev/${discodest}$((PARTROOT+1)) ];then
    solosync=1
    echo "2) Solo sincronizar sin reparticionar ${discodest}"
fi
echo "*) Salir"
echo -n "Opcion: "

read opcion
if  [ "$opcion" == "1" ]; then
    echo "Ejecutar el clon"
    echo
    echo "Copiando tabla de particiones"
    if [ $PARTROOT -eq 4 ] && [ -x /usr/sbin/sgdisk ];then
        #borro los posibles rastros de particiones gpt y mbr
        /usr/sbin/sgdisk --zap-all /dev/${discodest} >/dev/null 2>&1
    fi
    dd if=/dev/${discoorig} of=/dev/${discodest} bs=512 count=2048
    checkerr $?
    flush
    if [ -z "$EXPANDPART" ] || [ 0$EXPANDPART -ne 0 ];then
        echo "Particionando el disco:"
        particionar
        flush
    fi
    swapoff /dev/${discodest}${PARTSWAP} >/dev/null 2>&1
    echo "Recargo tabla de particiones"
    sync
    hdparm -z /dev/${discodest} || (flush;sleep 8;hdparm -z /dev/${discodest})
    checkerr $?
    flush
    echo "Genero swap"
    if [ 0$CLONAR_UUID ];then
        uuidswap=$( getuuid /dev/${discoorig}${PARTSWAP} )
    fi
    if [ -n "$uuidswap" ];then
        uuidswap="-U $uuidswap"
    fi
    mkswap -L SWAP-sda${PARTSWAP} ${uuidswap} /dev/${discodest}${PARTSWAP}
    echo "Clono ${discodest}${PARTBOOT}"
    mount -o remount,ro /dev/${discoorig}${PARTBOOT}
    dd if=/dev/${discoorig}${PARTBOOT} of=/dev/${discodest}${PARTBOOT} bs=1M
    mount -o remount,rw /dev/${discoorig}${PARTBOOT}
    flush
    fsck -fyC0 /dev/${discodest}${PARTBOOT}
    flush
    $TUNEFS $FORCETUNEFS -U random /dev/${discodest}${PARTBOOT}
    echo "Formateo ${discodest}${PARTROOT}"
    $MKFS $FORCEMKFS -t ext4 -j -L /new$RANDOM /dev/${discodest}${PARTROOT}
    $TUNEFS -o acl,user_xattr /dev/${discodest}${PARTROOT}
    $TUNEFS -E hash_alg=$HASH_ALG /dev/${discodest}${PARTROOT}
    $TUNEFS -c -1 -i 0 /dev/${discodest}${PARTROOT}
    flush
fi
if  [ "$opcion" == "1" ] || [ "$opcion" == "2"  -a "$solosync" == "1" ]; then
    sincronizar
    if [ 0$CLONAR_UUID ];then
        clonaruuid ${discoorig} ${discodest} ${PARTBOOT}
        clonaruuid ${discoorig} ${discodest} ${PARTROOT}
    fi
    touch /var/log/adminbcm_fechaclon
    echo "Clon finalizado con exito"
    exit 0
fi
echo "Salir sin realizar el clon"
exit -1
